<?php

namespace App\Model;

use Nette;
use Nette\Database\Table\ActiveRow;

/**
 * Class for handling activities related to project
 * @package App\Model
 */
class ProjectManager {
    use Nette\SmartObject;

    private Nette\Database\Explorer $database;

    public function __construct(Nette\Database\Explorer $database) {
        $this->database = $database;
    }

    /**
     * Function to get list of mentored projects with detailed information about projects
     * @param int $mentor_id mentor user identifier
     * @return Nette\Database\ResultSet
     */
    public function getMentoredProjects(int $mentor_id) {
        return $this->database->query('select a.name as assignment_name, a.id as assignment_id,
            t.id as team_id, t.name as team_name, p.id as project_id, p.start_date,
            
            (select COUNT(*) from EXPERIENCE e where e.project_id = p.id and e.mentor_state = 0) as experience_red,
            (select COUNT(*) from EXPERIENCE e where e.project_id = p.id and e.mentor_state = 1) as experience_orange,
            (select COUNT(*) from EXPERIENCE e where e.project_id = p.id and e.mentor_state = 2) as experience_green,
            (select COUNT(*) from EXPERIENCE e where e.project_id = p.id and e.mentor_state = 3) as experience_gray,
            
            (select COUNT(*) from REQUIREMENT_STATE s where s.project_id = p.id and s.mentor_state = 2) as requirement_green,
            (select COUNT(*) from REQUIREMENT_STATE s where s.project_id = p.id and s.mentor_state = 1) as requirement_orange,
            (select COUNT(*) from REQUIREMENT_STATE s where s.project_id = p.id and s.mentor_state = 3) as requirement_gray,
            (select (COUNT(*) - requirement_green - requirement_orange - requirement_gray) from REQUIREMENT s) as requirement_red,
            
            (select COUNT(*) from PROGRESS_STATE s where s.project_id = p.id and s.mentor_state = 2) as progress_green,
            (select COUNT(*) from PROGRESS_STATE s where s.project_id = p.id and s.mentor_state = 1) as progress_orange,
            (select COUNT(*) from PROGRESS_STATE s where s.project_id = p.id and s.mentor_state = 3) as progress_gray,
            (select (COUNT(*) - progress_green - progress_orange - progress_gray) from PROGRESS s) as progress_red

            from PROJECT p 
            inner join ASSIGNMENT a on a.id = p.assignment_id 
            inner join TEAM t on t.id = p.team_id
            where a.user_id = ?', $mentor_id);
    }

    /**
     * Function to load project information
     * @return Nette\Database\ResultSet
     */
    public function getProjectsWithInfo() {
        return $this->database->query('select a.name as assignment_name, a.id as assignment_id,
            t.id as team_id, t.name as team_name, p.id as project_id, p.start_date,
            
            (select COUNT(*) from EXPERIENCE e where e.project_id = p.id and e.mentor_state = 0) as experience_red,
            (select COUNT(*) from EXPERIENCE e where e.project_id = p.id and e.mentor_state = 1) as experience_orange,
            (select COUNT(*) from EXPERIENCE e where e.project_id = p.id and e.mentor_state = 2) as experience_green,
            (select COUNT(*) from EXPERIENCE e where e.project_id = p.id and e.mentor_state = 3) as experience_gray,
            
            (select COUNT(*) from REQUIREMENT_STATE s where s.project_id = p.id and s.mentor_state = 2) as requirement_green,
            (select COUNT(*) from REQUIREMENT_STATE s where s.project_id = p.id and s.mentor_state = 1) as requirement_orange,
            (select COUNT(*) from REQUIREMENT_STATE s where s.project_id = p.id and s.mentor_state = 3) as requirement_gray,
            (select (COUNT(*) - requirement_green - requirement_orange - requirement_gray) from REQUIREMENT s) as requirement_red,
            
            (select COUNT(*) from PROGRESS_STATE s where s.project_id = p.id and s.mentor_state = 2) as progress_green,
            (select COUNT(*) from PROGRESS_STATE s where s.project_id = p.id and s.mentor_state = 1) as progress_orange,
            (select COUNT(*) from PROGRESS_STATE s where s.project_id = p.id and s.mentor_state = 3) as progress_gray,
            (select (COUNT(*) - progress_green - progress_orange - progress_gray) from PROGRESS s) as progress_red

            from PROJECT p 
            inner join ASSIGNMENT a on a.id = p.assignment_id 
            right join TEAM t on t.id = p.team_id');
    }

    /**
     * Function returns assignments that is not assigned to any team but is mentored by given mentor
     * @param int $mentor_id mentor user identifier
     * @return Nette\Database\ResultSet
     */
    public function getMentoredProjectsNotAssigned(int $mentor_id) {
        return $this->database->query("select a.id, a.name, c.firstname, c.lastname from ASSIGNMENT a inner join CONTACT c on a.contact_id = c.id 
            where a.id not in (select p.assignment_id from PROJECT p) and a.user_id = ?", $mentor_id);
    }

    /**
     * Function return project record of given identifier
     * @param int $id project identifier
     * @return ActiveRow|null
     */
    public function getProject(int $id) {
        return $this->database->table("PROJECT")->get($id);
    }

    /**
     * Function returns project table content
     * @return Nette\Database\Table\Selection
     */
    public function getProjects() {
        return $this->database->table("PROJECT");
    }

    /**
     * Function returns referred team record
     * @param ActiveRow $project
     * @return ActiveRow|null
     */
    public function getTeamRelToProject(ActiveRow $project) {
        if($project == null) {
            return null;
        }

        return $project->ref('TEAM');
    }

    /**
     * Function to get team other links
     * @param int $projectId project identifier
     * @return array
     */
    public function getOtherLinks(int $projectId){
        $links = [];
        $prj = $this->getProject($projectId);

        $rows = $prj["other_links"];
        $rows = explode("\n",$rows);

        foreach ($rows as $row) {
            if (!empty(trim($row))) {
                $arr = explode(";", $row);
                if(count($arr)==2){
                    $links[$arr[0]]= $arr[1];
                }
            }
        }

        return $links;
    }


    /**
     * Function returns assignment related to given project
     * @param ActiveRow $project project record
     * @return ActiveRow|null
     */
    public function getAssignmentRelToProject(ActiveRow $project)
    {
        return $project->ref('ASSIGNMENT');
    }

    /**
     * Function returns mentor user that mentor give project
     * @param ActiveRow $project project record
     * @return ActiveRow|null
     */
    public function getMentorRelToProject(ActiveRow $project)
    {
        $assignment = $this->getAssignmentRelToProject($project);
        return $assignment->ref('USER', 'user_id')->ref('CONTACT');
    }

    /**
     * Function returns selection of experience related to given project
     * @param ActiveRow $project project record
     * @return Nette\Database\Table\GroupedSelection
     */
    public function getExperiencesRelToProject(ActiveRow $project) {
        return $project->related('EXPERIENCE');
    }

    /**
     * Function to check if user is mentoring given project
     * @param int $project project identifier
     * @param int $mentor mentor user identifier
     * @return bool
     */
    public function isProjectMentoredBy(int $project, int $mentor):bool {

        $assignment = $this->getAssignmentRelToProject($this->getProject($project));

        if($assignment == NULL) return false;

        if($assignment->toArray()['user_id'] == $mentor) {
            return true;
        }

        return false;
    }

    /**
     * Function to check if user is leader of given project
     * @param int $project project identifier
     * @param int $leader mentor user identifier
     * @return bool
     */
    public function isProjectLedBy(int $project, int $leader):bool {
        $team = $this->getTeamRelToProject($this->getProject($project));

        if($team == NULL) return false;

        if($team->toArray()['student_id'] == $leader) {
           return true;
        }

        return false;
    }
}