<?php

namespace App\Model;

use Nette;
use Nette\Database\Table\ActiveRow;
use Nette\Http\FileUpload;

/**
 * Class for handling activities related to students
 * @package App\Model
 */
class StudentManager {

    use Nette\SmartObject;

    private Nette\Database\Explorer $database;

    public function __construct(Nette\Database\Explorer $database) {

        $this->database = $database;
    }

    /**
     * Function returns all students
     * @return Nette\Database\Table\Selection
     */
    public function getAllStudents() {
        return $this->database->table('STUDENT');
    }

    /**
     * Function returns selection of students without team in array
     * @return array students without team (pairs id and student info)
     */
    public function getStudentsWithoutTeam() {
        return $this->getAllStudents()->where("team_id IS NULL")
            ->select('id, CONCAT(firstname," ",lastname," (",personalID,")") AS studentInfo')
            ->order('lastname ASC')->fetchPairs("id", 'studentInfo');
    }

    /**
     * Function to get student record
     * @param int $id student identifier
     * @return ActiveRow
     */
    public function getStudent(int $id): ActiveRow {
        return $this->database->table('STUDENT')->get($id);
    }

    /**
     * Function returns team related to given student record
     * @param ActiveRow $student student record
     * @return ActiveRow
     */
    public function teamRelToStudent(ActiveRow $student): ActiveRow {
        return $student->ref('TEAM');
    }

    /**
     * Function returns project record related to student
     * @param ActiveRow $student student record
     * @return Nette\Database\Table\GroupedSelection
     */
    public function projectRelToStudent(ActiveRow $student) {
        return $this->teamRelToStudent($student)->related('PROJECT');
    }

    /**
     * Function to get student record from user record
     * @param $userId user identifier
     * @return ActiveRow|null
     */
    public function getStudentByUser($userId): ActiveRow|null {
        return $this->database->table('USER')->get($userId)->ref('STUDENT');
    }

    /**
     * Function to remove student record
     * @param int $id student identifier
     * @return bool
     */
    public function deleteStudent(int $id):bool {
        $student = $this->getStudent($id);

        if(!$student) {
            return false;
        }

        $student->delete();

        return true;
    }

    /**
     * Function to create new student
     * @param $login students login
     * @param $firstname students first name
     * @param $lastname students last name
     * @param $personalId students personal ID
     * @param $email students contact email
     * @param $fieldOfStudy students field of study
     * @param $faculty students faculty
     * @param $period period TSP cycle identifier
     */
    public function createStudent($login, $firstname, $lastname, $personalId, $email, $fieldOfStudy, $faculty, $period) {
        $this->database->table('STUDENT')->insert([
            'login' => $login,
            'firstname' => $firstname,
            'lastname' => $lastname,
            'personalID' => $personalId,
            'email' => $email,
            'field_of_study' => $fieldOfStudy,
            'faculty' => $faculty,
            'period_id' => $period,
        ]);
    }

    /**
     * Function to create list of users from given file
     * @param $period
     * @param FileUpload $file
     */
    public function importStudents($period, FileUpload $file) {

        //$data = self::csv_in_array($file->temporaryFile, ";", "\"", true);
        $f = self::utf8_fopen_read($file->temporaryFile);
        $data = self::csvToArray($f);

        $insertData = array();
        foreach ($data as $item) {
            $insertData[] = [
                'period_id' => $period,
                'login' => $item['userName'],
                'firstname' => $item['jmeno'],
                'lastname' => $item['prijmeni'],
                'personalID' => $item['osCislo'],
                'email' => $item['email'],
                'field_of_study' => $item['nazevSp'],
                'faculty' => $item['fakultaSp'],
            ];
        }

        $this->database->table('STUDENT')->insert($insertData);
    }

    /**
     * Function to parse csv file to array
     * @param $file_to_read
     * @return array
     */
    private static function csvToArray($file_to_read) {

        $result = array();
        $header = fgetcsv($file_to_read, 1000, ';', "\"");

        while (!feof($file_to_read)) {
            $line = fgetcsv($file_to_read, 1000, ';', "\"");
            $rowArray = array();
            if (!$line) break;
            foreach ($line as $key => $value) {
                $rowArray[$header[$key]] = $value;
            }
            $result[] = $rowArray;
        }

        fclose($file_to_read);
        return $result;
    }

    /**
     * Function to open file with list of students
     * @param $fileName path to file
     * @return false|resource
     */
    private static function utf8_fopen_read($fileName) {
        $fc = iconv('windows-1250', 'utf-8', file_get_contents($fileName));
        $handle = fopen("php://memory", "rw");
        fwrite($handle, $fc);
        fseek($handle, 0);
        return $handle;
    }
}