<?php

namespace App\Model;

use Nette;
use Nette\Database\Table\ActiveRow;

/**
 * Class for handling activities related to users
 * @package App\Model
 */
class UserManager {

    private MailManager $mailManager;

    public function __construct(Nette\Database\Explorer $database, Nette\Security\Passwords $passwords, MailManager $mailManager) {
        $this->database = $database;
        $this->passwords = $passwords;
        $this->mailManager = $mailManager;
    }

    /**
     * Function returns IP address of connected client
     * @return string
     */
    public static function getIp(): string
    {
        if (isset($_SERVER['HTTP_CF_CONNECTING_IP'])) { // Cloudflare support
            $ip = $_SERVER['HTTP_CF_CONNECTING_IP'];
        } elseif (isset($_SERVER['REMOTE_ADDR']) === true) {
            $ip = $_SERVER['REMOTE_ADDR'];
            if (preg_match('/^(?:127|10)\.0\.0\.[12]?\d{1,2}$/', $ip)) {
                if (isset($_SERVER['HTTP_X_REAL_IP'])) {
                    $ip = $_SERVER['HTTP_X_REAL_IP'];
                } elseif (isset($_SERVER['HTTP_X_FORWARDED_FOR'])) {
                    $ip = $_SERVER['HTTP_X_FORWARDED_FOR'];
                }
            }
        } else {
            $ip = '127.0.0.1';
        }
        if (in_array($ip, ['::1', '0.0.0.0', 'localhost'], true)) {
            $ip = '127.0.0.1';
        }
        $filter = filter_var($ip, FILTER_VALIDATE_IP, FILTER_FLAG_IPV4);
        if ($filter === false) {
            $ip = '127.0.0.1';
        }

        return $ip;
    }

    /**
     * User registration
     * @param $username login of new user
     * @param $password password of new user
     * @param $email email of new user
     * @return array|bool|int|iterable|ActiveRow|Nette\Database\Table\Selection|\Traversable
     */
    public function register($username, $password, $email) {
        return $this->database->table('USER')->insert([
            'email' => $email,
            'username' => $username,
            'password' => $this->passwords->hash($password),
        ]);
    }

    /**
     * Function that check if user is in given role
     * @param $user
     * @param $role
     * @return bool
     */
    public function isInRole($user, $role):bool {
        $result = $this->database->table('USER')->get($user)->ref('ROLE');

        if($result->name == $role) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * Procedure that provides forgot password operation
     * @param $email recovery email
     */
    public function forgetPassword($email) {

        $token = Utils::randomString('0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ',64);

        $this->database->beginTransaction();

        $email = $this->database->table('USER')->where('email', $email);

        if(count($email) > 0) {
            while($this->database->table('USER')->where('recover_token', $token)->count() != 0) {
                $token = Utils::randomString('0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ',64);
            }

            $datetime = new Nette\Utils\DateTime();
            date_add($datetime, date_interval_create_from_date_string('6 hours'));

            $data = $email->fetch();

            $this->database->table('USER')->where('id', $data->id)->update([
                'recover_token' => $token,
                'recover_token_valid_time' => $datetime,
            ]);

            $this->database->commit();
            $this->mailManager->sendForgetEmail($data->email, $token);
        } else {
            $this->database->commit();
        }

    }

    /**
     * Function to validate recovery token
     * @param $token recovery token
     * @return bool
     */
    public function isRecoveryTokenValid($token) {
        $query = $this->database->table('USER')->where('recover_token', $token)->where('recover_token_valid_time >= current_timestamp()');

        return $query->count() > 0;
    }

    /**
     * Function to return all valid mentors
     * @return Nette\Database\Table\Selection
     */
    public function getAllMentors() {
        return $this->database->table('USER')->where('role_id', RoleManager::$ROLE_MENTOR_ID);
    }

    /**
     * Function to return all valid leaders
     * @return Nette\Database\Table\Selection
     */
    public function getAllLeaders() {
        return $this->database->table('USER')->where('role_id', RoleManager::$ROLE_LEADER_ID);
    }

    /**
     * Remove access privileges to given user
     * @param $userId user identifier
     * @return Nette\Database\ResultSet
     */
    public function disableLoginUser($userId) {
        return $this->database->query('UPDATE USER SET', ['can_login' => 0], 'WHERE id = ?', $userId);
    }

    /**
     * Create user as leader
     * @param $login login of new user
     * @param $password password of new user
     * @param $studentId identifier of student
     * @param $email contact email
     */
    public function createStudentUser($login, $password, $studentId, $email) {
        $this->createUser($login, $password, $studentId, $email, RoleManager::$ROLE_LEADER_ID);
    }

    /**
     * Create user as mentor
     * @param $login login of new user
     * @param $password password of new user
     * @param $email contact email
     * @param $firstname firstname
     * @param $lastname lastname
     */
    public function createMentorUser($login, $password, $email, $firstname, $lastname) {

        $user = $this->database->table('USER')->where('login', $login)->fetch();

        if(!$user) {

            $contact = $this->database->table('CONTACT')->insert([
                'firstname' => $firstname,
                'lastname' => $lastname,
                'email' => $email
            ]);

            $this->createUser($login, $password, $contact->id, $email, RoleManager::$ROLE_MENTOR_ID);
        } else {

            $user->contact->update([
                'firstname' => $firstname,
                'lastname' => $lastname,
                'email' => $email
            ]);

            $user->update([
                'email' => $email,
                'password' => $this->passwords->hash($password),
                'can_login' => 1,
                'role_id' => RoleManager::$ROLE_MENTOR_ID,
            ]);
        }
    }

    /**
     * Create new user
     * @param $login login of new user
     * @param $password password of new user
     * @param $contact contact identifier
     * @param $email contact email
     * @param $roleId role identifier
     * @return ActiveRow|null
     */
    public function createUser($login, $password, $contact, $email, $roleId):Nette\Database\Table\ActiveRow|null {

        $contact_col = "";
        if($roleId == RoleManager::$ROLE_LEADER_ID) {
            $contact_col = "student_id";
        } else if($roleId == RoleManager::$ROLE_MENTOR_ID) {
            $contact_col = "contact_id";
        } else if($roleId == RoleManager::$ROLE_GARANT_ID) {
            $contact_col = "contact_id";
        }

        $user = $this->database->table('USER')->insert([
            "login" => $login,
            "password" => $this->passwords->hash($password),
            "email" => $email,
            "role_id" => $roleId,
            $contact_col => $contact,
        ]);

        return $user;
    }

    /**
     * Password recover procedure
     * @param string $token recovery token
     * @param string $password new password
     */
    public function recoverPassword(string $token, string $password) {

        $this->database->beginTransaction();

        if($this->isRecoveryTokenValid($token)) {
            $this->getUserByRecoverToken($token)->update([
                'recover_token' => null,
                'recover_token_valid_time' => null,
                'password' => $this->passwords->hash($password),
            ]);
        }

        $this->database->commit();
    }

    /**
     * Function to get user that has given recovery token
     * @param $token recovery token
     * @return ActiveRow|null
     */
    public function getUserByRecoverToken($token):Nette\Database\Table\ActiveRow|null {
        return $this->database->table('USER')->where('recover_token', $token)->fetch();
    }

    /**
     * Get user record with given identifier
     * @param $userId user identifier
     * @return ActiveRow|null
     */
    public function getUser($userId):ActiveRow|null {
        return $this->database->table('USER')->get($userId);
    }

    /**
     * Function to check if given user has given password
     * @param ActiveRow $user user record
     * @param $password password
     * @return bool
     */
    public function verifyPassword(ActiveRow $user, $password):bool {

        if(!$user) {
            return false;
        }

        return $this->passwords->verify($password, $user->password);
    }

    /**
     * Procedure to change password if the old password match actual password
     * @param $userId user identifier
     * @param $oldPassword old password
     * @param $newPassword new password
     * @return bool
     */
    public function changePasswordIfPasswordValid($userId, $oldPassword, $newPassword):bool {

        $result = false;
        $this->database->beginTransaction();

        $user = $this->getUser($userId);
        if($this->verifyPassword($user, $oldPassword)) {
            $user->update([
                'password' => $this->passwords->hash($newPassword),
            ]);
            $result = true;
        }

        $this->database->commit();

        return $result;
    }

    /**
     * Function to get referenced contact information
     * @param ActiveRow|null $user
     * @return ActiveRow|null
     */
    public function getContact(ActiveRow|null $user):ActiveRow|null {
        return $user->ref('CONTACT');
    }

    /**
     * Function to update contact information
     * @param ActiveRow|null $contact contact record
     * @param $firstname firstname
     * @param $lastname lastname
     * @param $email contact email
     * @param $phone contact phone
     * @param $info more informations in correct format
     */
    public function updateContact(ActiveRow|null $contact, $firstname, $lastname, $email, $phone, $info) {
        if(!$contact) {
            return;
        }

        $contact->update([
            'firstname' => $firstname,
            'lastname' => $lastname,
            'email' => $email,
            'phone' => $phone,
            'info' => $info,
        ]);
    }

}