<?php

namespace App\Model;


use Nette\InvalidArgumentException;

/**
 * Class for operating auxiliary activities
 * @package App\Model
 */
class Utils {

    /**
     * String validation if entered link is URL address
     * @param string $url given link to validate
     * @return bool
     */
    public static function isStringUrl(string $url): bool {
        if (empty($url)) {
            return false;
        }

        if (!filter_var($url, FILTER_VALIDATE_URL) === false) {
            return true;
        }
        return false;
    }

    /**
     * Validation of custom link string
     * @param string $projectOtherLinksText given string to validate
     * @return bool
     */
    public static function validateOtherProjectLinksFormat(string $projectOtherLinksText): bool {
        if (empty($projectOtherLinksText)) {
            return true;
        }

        $rows = explode("\n", $projectOtherLinksText);

        foreach ($rows as $row) {
            if (!empty(trim($row))) {
                $arr = explode(";", $row);
                if (count($arr) != 2) {
                    return false;
                }
                if (!self::isStringUrl($arr[1])) {
                    return false;
                }
            }
        }
        return true;
    }

    /**
     * Function creates random string from given characters of given length
     * @param string $permittedChars string of allowed characters for string creation
     * @param int $length length of resulted string
     * @return string
     */
    public static function randomString(string $permittedChars, $length = 16): string {
        $input_length = strlen($permittedChars);
        $random_string = '';
        for ($i = 0; $i < $length; $i++) {
            $random_character = $permittedChars[mt_rand(0, $input_length - 1)];
            $random_string .= $random_character;
        }
        return $random_string;
    }

    /**
     * Function that concat (two) names together with space character
     * @param string|null $fistName
     * @param string|null $lastName
     * @return string|null
     */
    public static function getValidNameFromArgs(?string $fistName, ?string $lastName): ?string {
        if (!empty($fistName) && !empty($lastName)) {
            return $fistName . ' ' . $lastName;
        } else if (!empty($fistName)) {
            return $fistName;
        } else if ($lastName) {
            return $lastName;
        }
        return null;
    }

    /**
     * Validation password requirements of given string
     * @param string $password entered password string
     * @return bool true = password meets password requirements, false = password doesn't meet password requirements
     */
    public static function isPasswordStrong(string $password): bool {
        // Validate password strength
        $uppercase = preg_match('@[A-Z]@', $password);
        $lowercase = preg_match('@[a-z]@', $password);
        $number = preg_match('@[0-9]@', $password);
        $length = 8;
        //$specialChars = preg_match('@[^\w]@', $password);

        if (!$uppercase || !$lowercase || !$number || strlen($password) < $length) {
            return false;
        }

        return true;
    }

    /**
     * Parse links data format to array
     * @param string $projectLinks string with links format
     * @return array array of links
     */
    public static function transformProjectLinksFromLineToArray(string $projectLinks): array {
        $links = [];
        $rows = explode("\n", $projectLinks);

        foreach ($rows as $row) {
            if (!empty(trim($row))) {
                $arr = explode(";", $row);
                if (count($arr) == 2) {
                    $links[$arr[0]] = $arr[1];
                }
            }
        }
        return $links;
    }

    /**
     * Function creates string with Student informations
     * @param string $firstName
     * @param string $lastName
     * @param string $personalID
     * @param string $leaderString
     * @return string
     */
    public static function transformStudentDataToString(string $firstName, string $lastName, string $personalID, string $leaderString): string {
        if(empty($firstName) || empty($lastName) || empty($personalID)){
            throw new InvalidArgumentException("Student must have filled firstName, lastName and personalID");
        }
        return $firstName . " " . $lastName . " (" . $personalID . ") " . $leaderString;
    }

    /**
     * Function to parse given array to another array
     * @param int $id
     * @param array $relevantItems
     * @param string $dbNameOfId
     * @param string $dbNameOfRelItems
     * @return array
     */
    public static function getArrayForDynamicInsert(int $id, array $relevantItems, string $dbNameOfId, string $dbNameOfRelItems): array {
        if($id < 0) {
            throw new InvalidArgumentException("Id must be greater than zero.");
        }

        if(empty($dbNameOfId) || empty($dbNameOfRelItems)) {
            throw new InvalidArgumentException("dbNameOfId and dbnameOfRelIds must be filled.");
        }

        $values = array();
        foreach ($relevantItems as $relevantItem) {
            $values[] = [
                $dbNameOfId => $id,
                $dbNameOfRelItems => $relevantItem,
            ];
        }
        return $values;
    }

    /**
     * Function that generate random string of given length that meets password requirement
     * @param int $length string length
     * @return string resulted string
     */
    public static function getGeneratedPassword(int $length = 12): string {

        do {
            $password = Utils::randomString(Constants::$PASSWORD_ALPHABET, $length);
            $small = preg_match("/[a-z]/", $password);
            $large = preg_match("/[A-Z]/", $password);
            $numbers = preg_match("/[0-9]/", $password);

        } while(!$small || !$large || !$numbers);

        return $password;
    }

}