<?php

namespace App\Presenters;

use App\Components\ProjectLinksForm;
use App\Model\DefenceManager;
use App\Model\ExperienceManager;
use App\Model\MailManager;
use App\Model\ProjectManager;
use App\Model\RoleManager;
use App\Model\StudentManager;
use Nette\Application\UI\Form;
use Nette;

/**
 * Presenter class to handle requests to page with project details
 */
class ProjectPresenter extends BasePresenter {

    public ProjectManager $projectManager;
    public Nette\Database\Explorer $database;
    public StudentManager $studentManager;
    public DefenceManager $defenceManager;
    public ExperienceManager $experienceManager;
    public MailManager $mailManager;

    /**
     * @var ProjectLinksForm
     */
    private ProjectLinksForm $projectLinksForm;

    public function __construct(\Nette\Database\Explorer $database,
                                ProjectManager $projectManager,
                                StudentManager $studentManager,
                                DefenceManager $defenceManager,
                                ExperienceManager $experienceManager,
                                MailManager $mailManager,
                                ProjectLinksForm $projectLinksForm) {
        parent::__construct();
        $this->projectManager = $projectManager;
        $this->database = $database;
        $this->studentManager = $studentManager;
        $this->defenceManager = $defenceManager;
        $this->experienceManager = $experienceManager;
        $this->projectLinksForm = $projectLinksForm;
        $this->mailManager = $mailManager;
    }

    public function startup() {
        parent::startup();

        if($this->getView() == 'default') {

            if (!$this->getUser()->isInRole(RoleManager::$ROLE_LEADER)) {
                $this->redirect('Homepage:default');
            }

            $project = $this->studentManager->projectRelToStudent($this->studentManager
                ->getStudent($this->getUser()->getIdentity()->contact_id))->fetch();

            $this->redirect('Project:show', $project['id']);
        }
    }

    public function checkPrivileges(int $id){
        if($this->user->isInRole(RoleManager::$ROLE_LEADER)) {
            $project = $this->studentManager->projectRelToStudent($this->studentManager
                ->getStudent($this->getUser()->getIdentity()->contact_id))->fetch();
            if ($project["id"] != $id) {
                return false;
            }
        }
        return true;
    }

    public function actionShow(int $id){
        if(!$this->checkPrivileges($id)){
            $this->redirect("Homepage:");
        }
    }

    public function renderShow(int $id):void {

        $project = $this->projectManager->getProject($id);
        $this->template->project = $project;

        $this->template->otherLinks = $this->projectManager->getOtherLinks($id);
        $this->template->team = $this->projectManager->getTeamRelToProject($project);
        $assignment = $this->projectManager->getAssignmentRelToProject($project);
        $this->template->assignment = $assignment;
        $this->template->mentor = $this->projectManager->getMentorRelToProject($project);
        $this->template->experiences = $this->projectManager->getExperiencesRelToProject($project);
        $this->template->defences = $this->defenceManager->getDefencesRelToProject($project)->order('attempt', 'DESC');
        $this->template->requirements = $this->database->query("SELECT r.id, r.name,
       (SELECT s.leader_state FROM REQUIREMENT_STATE s WHERE s.requirement_id = r.id AND s.project_id = ?) as leader_state,
       (SELECT s.mentor_state FROM REQUIREMENT_STATE s WHERE s.requirement_id = r.id AND s.project_id = ?) as mentor_state
        FROM REQUIREMENT r order by r.order desc, r.id asc", $project->id, $project->id);
        $this->template->progress = $this->database->query("SELECT r.id, r.name,
       (SELECT s.leader_state FROM PROGRESS_STATE s WHERE s.progress_id = r.id AND s.project_id = ?) as leader_state,
       (SELECT s.mentor_state FROM PROGRESS_STATE s WHERE s.progress_id = r.id AND s.project_id = ?) as mentor_state
        FROM PROGRESS r order by r.order desc, r.id asc", $project->id, $project->id);

        if(!$this->getUser()->isInRole(RoleManager::$ROLE_GARANT)) {
            $tmp = $this->projectManager->isProjectMentoredBy($id, $this->getUser()->getIdentity()->getId());
            $tmp = $tmp && ($this->getUser()->isInRole(RoleManager::$ROLE_GARANT) || $this->getUser()->isInRole(RoleManager::$ROLE_MENTOR));
            $this->template->isUserMentor = $tmp;
        } else {
            $this->template->isUserMentor = true;
        }

        $tmp = $this->projectManager->isProjectLedBy($id, $this->getUser()->getIdentity()->getData()['contact_id']);
        $tmp = $tmp && ($this->getUser()->isInRole(RoleManager::$ROLE_GARANT) || $this->getUser()->isInRole(RoleManager::$ROLE_LEADER));
        $this->template->isUserLeader = $tmp;
        //$this->template->isUserLeader = true;
    }

    public function actionEditLinks(int $id) {
        if(!$this->checkPrivileges($id) || $this->getUser()->isInRole(RoleManager::$ROLE_MENTOR)){
            $this->flashMessage("Na tuto akci nemáte oprávnění.");
            $this->redirect("Homepage:");
        }
    }
    public function renderEditLinks(int $id){
        $this->template->projectName =  $this->projectManager->getAssignmentRelToProject($this->projectManager->getProject($id))['name'];
    }

    public function createComponentEditLinksForm(){
        $projectId = $this->getParameter("id");
        if(!is_numeric($projectId)) {
            $projectId = -1;
        }
        $form = $this->projectLinksForm->create((int)$projectId);
        $project = $this->projectManager->getProject($projectId);
        $form->onSuccess[] = function (Form $form) use ($project) {
            $this->flashMessage("Úspěch;Odkazy úspěšně upraveny.", "bg-success");
            $this->loggerManager->get('default')->info('The team edited the links on the team page', [
                'user_identity' => $this->getUser()->getIdentity()->getData(),
                'project_id' => $project->id,
                'team_id' => $project->team->id,
                'team_name' => $project->team->name,
            ]);
            $this->redirect('Project:');

        };
        $form['cancel']->setHtmlAttribute('onclick', "window.location = '" . $this->link('Project:') . "';");
        return $form;
    }

    public function createComponentAddExperienceForm(string $name): Form {
        $form = new Form;
        $form->addText('subject', 'Zkratka předmětu')->setRequired();
        $form->addText('experienceUse', 'Použití zkušenosti')->setRequired();
        $form->addSubmit('send', "Odeslat")
        ->setHtmlId("frm-addExperienceForm-submit");

        $form->onSuccess[] = [$this, "addExperienceSucceeded"];

        return $form;
    }

    public function addExperienceSucceeded(Form $form, \stdClass $values) {

        if(!$this->getUser()->isInRole(RoleManager::$ROLE_LEADER)) {
            $this->flashMessage('Chyba;Validace požadavku hlásí chybu.', 'bg-danger');
            return;
        }

        $project_id = $this->getParameter('id');

        $student = $this->studentManager->getStudent($this->getUser()->getIdentity()->contact_id);

        $user_project = $this->studentManager->projectRelToStudent($student)->fetch()->id;

        if($user_project != $project_id) {
            $this->flashMessage('Chyba;Validace požadavku hlásí chybu.', 'bg-danger');
            return;
        }

        $this->experienceManager->addExperience($project_id, $values->subject, $values->experienceUse);

        $this->flashMessage("Úspěch;Plán pro využití zkušenosti z jiných předmětů byl zadán.", "bg-success");

        $this->loggerManager->get('default')->info("User has added a new use of the subject experience.",
            ["user_identity"=>$this->getUser()->getIdentity()->getData(), "experience" => $values->experienceUse]);

        $this->redirect('this');
    }

    public function handleRequirementMentorCheck($project, $requirement, $value) {

        if($value < 0 || $value > 3) {
            $this->flashMessage('Chyba;Validace požadavku hlásí chybu.', 'bg-danger');
            return;
        }

        if(!$this->getUser()->isInRole(RoleManager::$ROLE_GARANT) && !$this->getUser()->isInRole(RoleManager::$ROLE_MENTOR)) {
            $this->flashMessage('Chyba;Validace požadavku hlásí chybu.', 'bg-danger');
            return;
        }

        if($this->getUser()->isInRole(RoleManager::$ROLE_MENTOR)) {
            if(!$this->projectManager->isProjectMentoredBy($project, $this->getUser()->getIdentity()->getId())) {
                $this->flashMessage('Chyba;Validace požadavku hlásí chybu.', 'bg-danger');
                return;
            }
        }

        $this->database->beginTransaction();

        $this->database->query("INSERT INTO REQUIREMENT_STATE (project_id, requirement_id, mentor_state) VALUES(?, ?, ?)
        ON DUPLICATE KEY UPDATE mentor_state = ?", $project, $requirement, $value, $value);

        $this->database->commit();
        $this->flashMessage('Úspěch;Operace proběhla úspěšně.', 'bg-success');
        $this->loggerManager->get('default')->info('The mentor adjusted the status requirements', [
            'user_id' => $this->getUser()->getIdentity()->getData(),
            'project_id' => $project,
            'requirement_id' => $requirement,
            'mentor_state' => $value,
        ]);
        $this->redirect('this');
    }

    public function handleRequirementLeaderCheck($requirement, $project, $value) {

        if($value < 0 || $value > 3) {
            $this->flashMessage('Chyba;Validace požadavku hlásí chybu.', 'bg-danger');
            return;
        }

        if(!$this->getUser()->isInRole(RoleManager::$ROLE_GARANT) && !$this->getUser()->isInRole(RoleManager::$ROLE_LEADER)) {
            $this->flashMessage('Chyba;Validace požadavku hlásí chybu.', 'bg-danger');
            return;
        }

        if($this->getUser()->isInRole(RoleManager::$ROLE_LEADER)) {
            if(!$this->projectManager->isProjectLedBy($project, $this->getUser()->getIdentity()->getData()['contact_id'])) {
                $this->flashMessage('Chyba;Validace požadavku hlásí chybu.', 'bg-danger');
                return;
            }
        }

        $this->database->beginTransaction();

        $this->database->query("INSERT INTO REQUIREMENT_STATE (project_id, requirement_id, leader_state) VALUES(?, ?, ?)
        ON DUPLICATE KEY UPDATE leader_state = ?", $project, $requirement, $value, $value);

        $this->database->commit();
        $this->flashMessage('Úspěch;Operace proběhla úspěšně.', 'bg-success');
        $this->loggerManager->get('default')->info('The leader adjusted the status requirements', [
            'user_id' => $this->getUser()->getIdentity()->getData(),
            'project_id' => $project,
            'requirement_id' => $requirement,
            'leader_state' => $value,
        ]);
        $this->redirect('this');
    }



    public function handleProgressMentorCheck($project, $progress, $value) {

        if($value < 0 || $value > 3) {
            $this->flashMessage('Chyba;Validace požadavku hlásí chybu.', 'bg-danger');
            return;
        }

        if(!$this->getUser()->isInRole(RoleManager::$ROLE_GARANT) && !$this->getUser()->isInRole(RoleManager::$ROLE_MENTOR)) {
            $this->flashMessage('Chyba;Validace požadavku hlásí chybu.', 'bg-danger');
            return;
        }

        if($this->getUser()->isInRole(RoleManager::$ROLE_MENTOR)) {
            if(!$this->projectManager->isProjectMentoredBy($project, $this->getUser()->getIdentity()->getId())) {
                $this->flashMessage('Chyba;Validace požadavku hlásí chybu.', 'bg-danger');
                return;
            }
        }

        $this->database->beginTransaction();

        $this->database->query("INSERT INTO PROGRESS_STATE (project_id, progress_id, mentor_state) VALUES(?, ?, ?)
        ON DUPLICATE KEY UPDATE mentor_state = ?", $project, $progress, $value, $value);

        $this->database->commit();
        $this->flashMessage('Úspěch;Operace proběhla úspěšně.', 'bg-success');
        $this->loggerManager->get('default')->info('The mentor adjusted the status progress', [
            'user_id' => $this->getUser()->getIdentity()->getData(),
            'project_id' => $project,
            'progress_id' => $progress,
            'mentor_state' => $value,
        ]);
        $this->redirect('this');
    }

    public function handleProgressLeaderCheck($progress, $project, $value) {

        if($value < 0 || $value > 3) {
            $this->flashMessage('Chyba;Validace požadavku hlásí chybu.', 'bg-danger');
            return;
        }

        if(!$this->getUser()->isInRole(RoleManager::$ROLE_GARANT) && !$this->getUser()->isInRole(RoleManager::$ROLE_LEADER)) {
            $this->flashMessage('Chyba;Validace požadavku hlásí chybu.', 'bg-danger');
            return;
        }

        if($this->getUser()->isInRole(RoleManager::$ROLE_LEADER)) {
            if(!$this->projectManager->isProjectLedBy($project, $this->getUser()->getIdentity()->getData()['contact_id'])) {
                $this->flashMessage('Chyba;Validace požadavku hlásí chybu.', 'bg-danger');
                return;
            }
        }

        $this->database->beginTransaction();

        $this->database->query("INSERT INTO PROGRESS_STATE (project_id, progress_id, leader_state) VALUES(?, ?, ?)
        ON DUPLICATE KEY UPDATE leader_state = ?", $project, $progress, $value, $value);

        $this->database->commit();
        $this->flashMessage('Úspěch;Operace proběhla úspěšně.', 'bg-success');
        $this->loggerManager->get('default')->info('The leader adjusted the status progress', [
            'user_id' => $this->getUser()->getIdentity()->getData(),
            'project_id' => $project,
            'progress_id' => $progress,
            'leader_state' => $value,
        ]);
        $this->redirect('this');
    }

    public function handleExperienceMentorCheck($experience, $project, $value) {

        if($value < 0 || $value > 3) {
            $this->flashMessage('Chyba;Validace požadavku hlásí chybu.', 'bg-danger');
            return;
        }

        if(!$this->getUser()->isInRole(RoleManager::$ROLE_GARANT) && !$this->getUser()->isInRole(RoleManager::$ROLE_MENTOR)) {
            $this->flashMessage('Chyba;Validace požadavku hlásí chybu.', 'bg-danger');
            return;
        }

        if($this->getUser()->isInRole(RoleManager::$ROLE_MENTOR)) {
            if(!$this->projectManager->isProjectMentoredBy($project, $this->getUser()->getIdentity()->getId())) {
                $this->flashMessage('Chyba;Validace požadavku hlásí chybu.', 'bg-danger');
                return;
            }
        }

        $this->database->beginTransaction();

        $this->database->query("UPDATE EXPERIENCE SET mentor_state = ? WHERE id = ?", $value, $experience);

        $this->database->commit();

        $this->flashMessage('Úspěch;Operace proběhla úspěšně.', 'bg-success');

        $this->loggerManager->get('default')->info('The leader adjusted the status experience', [
            'user_id' => $this->getUser()->getIdentity()->getData(),
            'project_id' => $project,
            'requirement_id' => $experience,
            'mentor_state' => $value,
        ]);
        $this->redirect('this');
    }

    public function handleExperienceLeaderCheck($experience, $project, $value) {

        if($value < 0 || $value > 3) {
            $this->flashMessage('Chyba;Validace požadavku hlásí chybu.', 'bg-danger');
            return;
        }

        if(!$this->getUser()->isInRole(RoleManager::$ROLE_GARANT) && !$this->getUser()->isInRole(RoleManager::$ROLE_LEADER)) {
            $this->flashMessage('Chyba;Validace požadavku hlásí chybu.', 'bg-danger');
            return;
        }

        if($this->getUser()->isInRole(RoleManager::$ROLE_LEADER)) {
            if(!$this->projectManager->isProjectLedBy($project, $this->getUser()->getIdentity()->getData()['contact_id'])) {
                $this->flashMessage('Chyba;Validace požadavku hlásí chybu.', 'bg-danger');
                return;
            }
        }

        $this->database->beginTransaction();
        $this->database->query("UPDATE EXPERIENCE SET leader_state = ? WHERE id = ?", $value, $experience);

        $this->database->commit();
        $this->flashMessage('Úspěch;Operace proběhla úspěšně.', 'bg-success');

        $this->loggerManager->get('default')->info('The leader adjusted the status experience', [
            'user_id' => $this->getUser()->getIdentity()->getData(),
            'project_id' => $project,
            'requirement_id' => $experience,
            'leader_state' => $value,
        ]);
        $this->redirect('this');
    }

    public function canAccessPage(): bool {

        if(!$this->getUser()->isLoggedIn()) {
            return false;
        }

        $projectId = intval($this->getParameter('id'));

        if($this->getUser()->isInRole(RoleManager::$ROLE_GARANT)) {
            return true;
        }

        if($this->getUser()->isInRole(RoleManager::$ROLE_MENTOR)) {
            return $this->projectManager->isProjectMentoredBy($projectId, $this->getUser()->getId());
        }

        if($this->getUser()->isInRole(RoleManager::$ROLE_LEADER)) {
            //return $this->projectManager->isProjectLedBy($projectId, $this->getUser()->getId());
            return true;
        }
        return true;
    }

}