<?php

namespace App\Presenters;

use App\Components\SignForgetForm;
use App\Components\SignInForm;
use App\Model\UserManager;
use App\Model\Utils;
use Contributte\Monolog\LoggerManager;
use Nette;
use Nette\Application\UI\Form;

/**
 * Presenter class to handle requests to sign pages
 */
class SignPresenter extends Nette\Application\UI\Presenter {

    private UserManager $userManager;


    /** @var LoggerManager **/
    private LoggerManager $loggerManager;

    /**
     * @var SignInForm
     */
    private SignInForm $signInForm;

    /**
     * @var SignForgetForm
     */
    private SignForgetForm $signForgetForm;

    public function __construct(UserManager $userManager, SignInForm $signInForm, SignForgetForm $signForgetForm) {
        parent::__construct();
        $this->userManager = $userManager;
        $this->signInForm = $signInForm;
        $this->signForgetForm = $signForgetForm;
    }

    public function startup() {
        parent::startup();
        if($this->getUser()->isLoggedIn() && $this->getRequest()->getParameter('action') != 'out') {
            $this->redirect("Homepage:");
        }
    }

    /**
     * DI injection of Logger manager
     * @param LoggerManager $loggerManager
     */
    public function injectLoggerManager(LoggerManager $loggerManager): void {
        $this->loggerManager = $loggerManager;
    }

    protected function createComponentSignInForm(): Form {

        $form = $this->signInForm->create();
        $form->onSuccess[] = function () {
            $this->loggerManager->get('default')->info('User logged in', ['user_identity' => $this->getUser()->getIdentity()->getData(), 'ip_addr' => UserManager::getIp()]);
            $this->redirect("Homepage:");
        };
        $form->onError[] = function() {
            $this->loggerManager->get('default')->info('Login failed', ['ip_addr' => UserManager::getIp()]);
            $this->flashMessage("Chyba;Přihlášení se nezdařilo. Zkontrolujte kombinaci přihlašovacího jmena a hesla", 'bg-warning');
        };
        return $form;
    }

    protected function createComponentSignForgetForm(): Form {
        $form = $this->signForgetForm->create();
        $form->onSuccess[] = [$this, 'forgetFormSucceeded'];

        $form->onError[] = function() {
            $this->flashMessage('Úspěch;Požadavek byl odeslán. Pokud se zadaný login shoduje, bude odeslán email s instrukcemi.', 'bg-success');
            $this->redirect('this');
        };

        return $form;
    }

    public function forgetFormSucceeded(Form $form, Nette\Utils\ArrayHash $values) {
        $this->loggerManager->get('default')->info('Password reset requested', ['ip_addr' => UserManager::getIp(), 'email' => $values->email]);
        $this->flashMessage('Úspěch;Požadavek byl odeslán. Pokud se zadaný login shoduje, bude odeslán email s instrukcemi.', 'bg-success');
        $this->redirect('this');
    }

    public function actionOut() {
        $this->loggerManager->get('default')->info('User logged out', ['user_identity' => $this->getUser()->getIdentity()->getData()]);
        $this->getUser()->logout();
        $this->flashMessage('Úspěch;Odhlášení bylo úspěšné.', 'bg-success');
        $this->redirect('Homepage:');
    }

    protected function createComponentSignUpForm(): Form {
        $form = new Form;

        $form->addText('username')->setRequired();

        $form->addEmail('email')->setRequired();

        $form->addPassword('password1', 'Heslo:')->setRequired()
            ->addRule($form::MIN_LENGTH, 'Heslo musí být dlouhé alespoň %d znaků.', 8)
            ->addRule($form::PATTERN, 'Heslo neobsahuje velké písmeno.', '\w*([A-Z])+\w*')
            ->addRule($form::PATTERN, 'Heslo neobsahuje číslici.', '\w*([0-9])+\w*')
            ->addRule($form::PATTERN, 'Heslo neobsahuje malé písmeno.', '\w*([a-z])+\w*');
        $form->addPassword('password2', 'Heslo znovu:')->setRequired('Vyplňte heslo');
        $form->addCheckbox('souhlas', 'Souhlas se zpracováním údajů')->setRequired('Vyplňte souhlas se zpracováním údajů.');
        $form->addSubmit('send', "Registrovat");

        $form->onSuccess[] = [$this, 'signUpFormSucceeded'];

        return $form;
    }

    public function signUpFormSucceeded(Form $form, \stdClass $values) {
        if($values->password1 != $values->password2) {
            $form->addError('Hesla se neshodují');
            return;
        }

        $this->userManager->register($values->username, $values->password1, $values->email);
        $this->loggerManager->get('default')->notice('The user has been registered',
            ['user_id' => $this->getUser()->getId(), 'user_identity' => $this->getUser()->getIdentity()->getData()]);

        $this->flashMessage("Registrace proběhla úspěšně, nyní se můžete přihlásit.");
        $this->redirect('HomePage:');

    }


}