<?php

declare(strict_types=1);

namespace App\Presenters;

use App\Components\StudentAdvancesForm;
use App\Components\TeamAdvancesForm;
use App\Components\TeamEditForm;
use App\Components\TeamMeetingInfoForm;
use App\Model\InterestManager;
use App\Model\MeetingFacade;
use App\Model\ProjectManager;
use App\Model\RoleManager;
use App\Model\StudentManager;
use App\Model\TeamManager;
use App\Model\TeamRoleManager;
use Nette;
use Nette\Application\UI\Form;
use Nette\Utils\DateTime;

/**
 * Presenter class to handle requests to page with team details
 */
class TeamPresenter extends BasePresenter {

    private MeetingFacade $meetings;

    private InterestManager $interestManager;

    public Nette\Database\Explorer $database;

    /**
     * @var TeamAdvancesForm
     */
    private TeamAdvancesForm $teamAdvancesForm;

    private TeamEditForm $teamEditForm;

    private StudentAdvancesForm $studentAdvancesForm;
    /**
     * @var TeamRoleManager
     */
    private TeamRoleManager $teamRoleManager;

    private ProjectManager $projectManager;

    private TeamMeetingInfoForm $teamMeetingInfoForm;

    /** @var array|string[] For meeting person translation */
    private array $notifyOptions = [
        'mentor' => 'Mentor',
        'creator' => "Zadavatel",
        'intern' => "Interní",
        'other' => "Jiné",
    ];

    public function __construct(\Nette\Database\Explorer $database,
                                TeamRoleManager          $teamRoleManager,
                                MeetingFacade            $meetings,
                                TeamAdvancesForm         $teamAdvancesForm,
                                TeamEditForm             $teamEditForm,
                                StudentAdvancesForm      $studentAdvancesForm,
                                TeamMeetingInfoForm      $teamMeetingInfoForm,
                                InterestManager          $interestManager,
                                ProjectManager           $projectManager
                                ) {

        parent::__construct();
        $this->database = $database;
        $this->meetings = $meetings;
        $this->interestManager = $interestManager;
        $this->teamRoleManager = $teamRoleManager;
        $this->teamAdvancesForm = $teamAdvancesForm;
        $this->teamEditForm = $teamEditForm;
        $this->studentAdvancesForm = $studentAdvancesForm;
        $this->teamMeetingInfoForm = $teamMeetingInfoForm;
        $this->projectManager = $projectManager;
    }

    public function startup() {
        parent::startup();

        if ($this->getView() == 'default') {

            if (!$this->getUser()->isInRole(RoleManager::$ROLE_LEADER)) {
                $this->redirect('Homepage:default');
            }

            $team = $this->database->table('USER')
                ->get($this->getUser()->getId())->ref('STUDENT')
                ->ref('TEAM');

            $this->redirect('Team:show', $team->id);
        }
    }

    public function checkPrivileges(int $id) {
        if ($this->user->isInRole(RoleManager::$ROLE_LEADER)) {
            $teamId = $this->teamManager->getTeamIdRelByStudent($this->getUser()->getIdentity()->contact_id);

            if ($teamId != $id) {
                return false;
            }
        }
        return true;
    }

    public function isMentorOfTheTeam(Nette\Database\Table\ActiveRow|null $team):bool {
        if($team == null) {
            return false;
        }

        if($this->getUser()->isInRole(RoleManager::$ROLE_MENTOR)) {
            $project = $team->related('PROJECT')->fetch();
            if($project->assignment->user->id == $this->getUser()->getId()) {
                return true;
            }
        }

        return false;
    }

    public function actionShow(int $id) {
        if (!$this->checkPrivileges($id)) {
            $this->redirect("Homepage:");
        }
    }

    public function renderShow(int $id): void {

        $teamLeader = $this->teamManager->getTeamLeader($id);
        $teamMembersExceptLeader = $this->teamManager->getTeamMembersExceptLeader($teamLeader->id, $id);
        $this->template->idTeam = $id;
        $this->template->team = $this->database->table('TEAM')->get($id);
        $teamMembers = $this->teamManager->getTeamMembers($id);
        $this->template->teamMembers = $teamMembers;//$this->template->team->related("STUDENT");
        $this->template->teamRoles = $this->teamRoleManager->getAllRolesForTeam($id);
        if($this->teamManager->hasTeamAssignmentAR($this->template->team)) {
            $this->template->assignment = $this->template->team
                ->related("PROJECT")->fetch()
                ->ref("ASSIGNMENT");
            $this->template->meetings = $this->template->team->related('PROJECT')->fetch()
                ->related("MEETING", 'project_id')->order('datetime DESC');
            $this->template->hasAssignment = true;
        } else {
            $this->template->hasAssignment = false;
        }

        //bdump($this->template->assignment);
        $this->template->preferences = $this->template->team
            ->related("TEAM_SKILL");
        $this->template->project = $this->teamManager->getProjectRelToTeam($this->template->team);
        $this->template->notifyConverter = $this->notifyOptions;

        $this->template->interests = $this->interestManager->getInterestsByTeam($this->template->team);

        $this->template->isLeader = $this->getUser()->isInRole(RoleManager::$ROLE_LEADER)
            && $this->template->team->student->id == $this->getUser()->getIdentity()->getData()['contact_id'];
        $this->template->isMentor = $this->isMentorOfTheTeam($this->template->team);

    }

    public function actionEditSkills(int $id) {
        if (!$this->checkPrivileges($id) || $this->getUser()->isInRole(RoleManager::$ROLE_MENTOR)) {
            $this->flashMessage("Varování;Na tuto akci nemáte oprávnění.", "bg-warning");
            $this->redirect("Homepage:");
        }
    }

    public function renderEditSkills(int $id): void {

        $this->template->team = $this->database->table('TEAM')->get($id);
    }

    public function actionEditMeetingTopic(int $id) {
        /*if (!$this->checkPrivileges($id) || $this->getUser()->isInRole(RoleManager::$ROLE_MENTOR)) {
            $this->flashMessage("Na tuto akci nemáte oprávnění.");
            $this->redirect("Homepage:");
        }*/
    }

    public function handleDeleteInterest($assignmentId, $teamId) {

        if($this->getUser()->isInRole(RoleManager::$ROLE_LEADER)) {
            $this->database->beginTransaction();
            $team = $this->studentManager->getStudentByUser($this->getUser()->getId())->ref('TEAM');
            if($team->related('ASSIGNMENT_INTEREST')->where('team_id', $teamId)->where('assignment_id', $assignmentId)->count() == 1) {
                $team->related('ASSIGNMENT_INTEREST')->where('team_id', $teamId)->where('assignment_id', $assignmentId)->delete();
                $this->loggerManager->get('default')->info('The team withdrew interest in the topic', [
                    'user_identity' => $this->getUser()->getIdentity()->getData(),
                    'team_id' => $teamId,
                    'assignment_id' => $assignmentId,
                ]);
                $this->flashMessage("Úspěch; Odstranění zájmu proběhlo úspěšně", 'bg-success');
            } else {
                $this->flashMessage("Chyba; Při vyřizování požadavku se vyskytla validační chyba.", 'bg-warning');
            }
            $this->database->commit();
            $this->redirect('this');

        } else {
            $this->flashMessage("Varování;Na tuto akci nemáte oprávnění.", 'bg-warning');
            $this->redirect("Homepage:");
        }
    }

    protected function createComponentEditMeetingTopicForm(): Form {
        $meetingId = intval($this->getParameter("id"));
        if (!is_numeric($meetingId)) {
            $teamId = -1;
        }
        $form = $this->teamMeetingInfoForm->create((int)$meetingId);

        $meeting = $this->meetings->getMeeting($meetingId);

        $form->onSuccess[] = function (Form $form) use ($meeting) {
            $this->loggerManager->get('default')->info('User edited meeting record', [
                'user_identity' => $this->getUser()->getIdentity()->getData(),
                'meeting_id' => $meeting->id,
                'team_id' => $meeting->project->team->id,
            ]);
            $this->flashMessage("Úspěch;Podrobný zápis byl úspěšně přidán.", "bg-success");
            $this->redirect('Team:show', $meeting->project->team->id);

        };

        $form['cancel']->setHtmlAttribute('onclick', "window.location = '" . $this->link('Team:') . "';");
        return $form;
    }

    protected function createComponentTeamAdvancesForm(): Form {
        $teamId = $this->getParameter("id");
        if (!is_numeric($teamId)) {
            $teamId = -1;
        }
        $form = $this->teamAdvancesForm->create((int)$teamId);
        $form->onSuccess[] = function (Form $form) use ($teamId) {
            $this->loggerManager->get('default')->info('User edited team profile information', [
                'user_identity' => $this->getUser()->getIdentity()->getData(),
                'team_id' => $teamId,
            ]);
            $this->flashMessage("Úspěch;Editace profilů týmu proběhla úspěšně.", "bg-success");
            $this->redirect('Team:');

        };


        return $form;
    }

    protected function createComponentStudentAdvancesForm(): Form {
        $teamId = $this->getParameter("id");
        if (!is_numeric($teamId)) {
            $teamId = -1;
        }
        $form = $this->studentAdvancesForm->create((int)$teamId);
        $form->onSuccess[] = function (Form $form) use ($teamId) {
            $this->loggerManager->get('default')->info('User edited detailed team students information', [
                'user_identity' => $this->getUser()->getIdentity()->getData(),
                'team_id' => $teamId,
            ]);
            $this->flashMessage("Úspěch;Editace profilů členů týmu proběhla úspěšně.", "bg-success");
            $this->redirect('Team:');

        };
        $form['cancel']->setHtmlAttribute('onclick', "window.location = '" . $this->link('Team:') . "';");
        return $form;
    }

    public function actionEdit(int $id) {
        if (!$this->checkPrivileges($id)) {
            $this->redirect("Homepage:");
        }
    }

    public function renderEdit(int $id): void {

        $this->template->team = $this->database->table('TEAM')->get($id);
    }

    public function actionEditPosition(int $id) {
        if (!$this->checkPrivileges($id) || $this->getUser()->isInRole(RoleManager::$ROLE_MENTOR)) {
            $this->flashMessage("Varování;Na tuto akci nemáte oprávnění.", "bg-warning");
            $this->redirect("Homepage:");

        }
    }

    public function renderEditPosition(int $id): void {

        $this->template->team = $this->database->table('TEAM')->get($id);
    }

    protected function createComponentTeamEditForm(): Form {
        $teamId = $this->getParameter("id");
        if (!is_numeric($teamId)) {
            $teamId = -1;
        }
        $form = $this->teamEditForm->create((int)$teamId);
        $form['cancel']->setHtmlAttribute('onclick', "window.location = '" . $this->link('Team:show', $teamId) . "';");
        $form->onSuccess[] = function (Form $form) use ($teamId) {
            $this->loggerManager->get('default')->info('User edited team information', [
                'user_identity' => $this->getUser()->getIdentity()->getData(),
                'team_id' => $teamId,
            ]);
            $this->flashMessage("Úspěch;Editace týmu proběhla úspěšně.", "bg-success");
            $teamId = $this->getParameter("id");
            $this->redirect('Team:show', $teamId);
        };
        return $form;
    }

    protected function createComponentAddMeetingForm(): Form {
        $form = new Form;

        $form->addText('date', "Datum a čas")
            ->setRequired("Termín schůzky musí být vyplněn.")
            ->setHtmlType("datetime-local")
        ->setHtmlAttribute("class","form-control");


        $form->addText('place', "Místnost")
            ->setRequired("Místo setkání musí být vyplněno.")
            ->setHtmlAttribute("placeholder", "Zadejte název místnosti")
        ->setHtmlAttribute("class","form-control");

        $form->addText('topic', "Téma (příp. poznámka):")
            ->setRequired("Obsah schůzky musí být vyplněn.")
            ->setHtmlAttribute("placeholder", "Poznámka")
            ->setHtmlAttribute("class","form-control");


        $notifyOptions = [
            'mentor' => 'Mentor',
            'creator' => "Zadavatel",
            'intern' => "Interní",
            'other' => "Jiné",
        ];

        $form->addSelect('notifyWho', 'S kým:', $notifyOptions)
            ->setHtmlAttribute("class","form-control");

        //$form->addCheckbox("linking", "Upozornit zúčastněné")
        //    ->setHtmlAttribute("class","form-control");

        $form->addSubmit('send', 'Naplánovat')
            ->setHtmlAttribute("class","form-control")
            ->setHtmlId("frm-AddMeetingForm-submit");

        $form->onSuccess[] = [$this, 'addMeetingFormSucceeded'];

        return $form;
    }

    public function addMeetingFormSucceeded(array $values): void {
        if (!$this->getUser()->isLoggedIn()) {
            $this->redirect("Sign:in");
        }

        $teamId = intval($this->getParameter('id'));

        $team = $this->teamManager->getTeamById($teamId);

        if(!$this->isUserTeamLeader($team['student_id']) && !$this->isMentorOfTheTeam($team)) {
            $this->flashMessage('Chyba;Nedostatečná oprávnění pro provedení této akce', 'bg-danger');
            return;
        }

        $meeting = $this->meetings->addMeeting($values['date'], $values['place'],
            $values['notifyWho'], $values['topic'],
            $this->teamManager->getProjectRelToTeam($team)->id,
            $values['notifyWho']);
        $this->loggerManager->get('default')->info('A new meeting has been created', [
            'user_identity' => $this->getUser()->getIdentity()->getData(),
            'meeting_id' => $meeting->id,
            'team_id' => $meeting->project->team->id,
        ]);
        $this->flashMessage("Úspěch;Schůzka byla přidána.", "bg-success");
        $this->redirect('Team:show', $teamId);


    }

    public function isUserTeamLeader($leader_id):bool {
        if($this->getUser()->isInRole(RoleManager::$ROLE_LEADER)) {
            if($this->getUser()->getIdentity()->getData()['contact_id'] == $leader_id) {
                return true;
            }
        }

        return false;
    }

    public function canAccessPage(): bool {
        if(!$this->getUser()->isLoggedIn()) {
            return false;
        }

        $action = $this->getRequest()->getParameter('action');

        if($action == 'editMeetingTopic') {

            $parameter = $this->getParameter('id');
            if($parameter == null) {
                return false;
            }

            $meet = $this->meetings->getMeeting(intval($parameter));

            if($meet == null) {
                return false;
            }

            $team = $meet->project->team;

            if ($this->getUser()->isInRole(RoleManager::$ROLE_LEADER)) {

                if ($team['student_id'] == $this->getUser()->getIdentity()->getData()['contact_id']) {
                    return true;
                }
            } else if($this->getUser()->isInRole(RoleManager::$ROLE_MENTOR) && $this->isMentorOfTheTeam($team)) {
                return true;
            }

        } else if($action == 'addMeetingTopic') {

        } else {

            $parameter = $this->getParameter('id');
            if ($parameter == null) {
                return true;
            }

            $teamId = intval($parameter);

            if ($this->getUser()->isInRole(RoleManager::$ROLE_LEADER)) {
                $team = $this->teamManager->getTeamById($teamId);

                if ($team['student_id'] == $this->getUser()->getIdentity()->getData()['contact_id']) {
                    return true;
                }
            } else if($this->getParameter('action') == 'show') {
                if ($this->getUser()->isInRole(RoleManager::$ROLE_MENTOR) || $this->getUser()->isInRole(RoleManager::$ROLE_GARANT)) {
                    return true;
                }
            }

        }

        return false;
    }

}