<?php

namespace App\Components;

use App\Model\AssignmentManager;
use App\Model\Constants;
use App\Model\RoleManager;
use App\Model\TagManager;
use App\Model\UserManager;
use App\Model\Utils;
use Nette;
use Nette\Application\UI\Form;
use Nette\Utils\ArrayHash;

/**
 * Form component class for creating assignments.
 * @package App\Components
 */
class AssignmentCreationForm extends Form {

    protected AssignmentManager $assignmentManager;
    protected UserManager $userManager;
    protected TagManager $tagManager;
    protected Nette\Database\Explorer $database;

    /**
     * Component constructor
     * @param AssignmentManager $assignmentManager
     * @param UserManager $userManager
     * @param TagManager $tagManager
     * @param Nette\Database\Explorer $database
     */
    public function __construct(AssignmentManager $assignmentManager,
                                UserManager $userManager,
                                TagManager $tagManager,
                                Nette\Database\Explorer $database) {
        parent::__construct();

        $this->assignmentManager = $assignmentManager;
        $this->userManager = $userManager;
        $this->tagManager = $tagManager;
        $this->database = $database;

    }

    /**
     * Factory function for creating assignment creation form
     * @param $userId the access user identifier for restricting access
     * @param int $assignmentId assignment identifier for specify editing assignment
     * @return Form resulted Form instance
     * @throws \Exception
     */
    public function create($userId, int $assignmentId = 0) {

        $defaults = [
            'name' => "",
            'description' => '',
            'team_size' => '',
            'duration' => '',
            'creatorFirstname' => '',
            'creatorLastname' => '',
            'creatorEmail' => '',
            'creatorPhone' => '',
            'creatorInfo' => '',
            'visibility' => Constants::$ASSIGNMENT_VISIBILITY_HIDDEN,
            'mentor' => 0,
            'profile' => array(),
        ];

        $form = new Form();

        if($assignmentId != 0) {
            $row = $this->assignmentManager->getAssignment($assignmentId);
            $assignmentTags = $this->assignmentManager->getAssignmentProfiles($row);

            if($row == null) {
                throw new \Exception("Cannot create form with invalid id");
            }

            $defaults = [
                'name' => $row->name,
                'description' => $row->description,
                'team_size' => $row->team_size,
                'duration' => $row->duration,
                'creatorFirstname' => $row->contact->firstname,
                'creatorLastname' => $row->contact->lastname,
                'creatorEmail' => $row->contact->email,
                'creatorPhone' => $row->contact->phone,
                'creatorInfo' => $row->contact->info,
                'visibility' => $row->hidden,
                'mentor' => ($row->user_id),
            ];
        }

        $form->addText('name', 'Název zadání')->setRequired('Název zadání musí být vyplněn.')
            ->setDefaultValue($defaults['name'])
            ->setHtmlAttribute("class","form-control");
        $form->addTextArea('description', 'Krátký popis zadání')
            ->setDefaultValue($defaults['description'])
            ->setHtmlAttribute('rows', "10")
            ->setHtmlAttribute("class","form-control");
        $form->addInteger('teamSize', 'Doporučená velikost týmu')
            ->setDefaultValue($defaults['team_size'])
            ->setHtmlAttribute("class","form-control");
        $form->addInteger('duration', 'Časová náročnost [člověko-hodin]')
            ->setDefaultValue($defaults['duration'])
            ->setHtmlAttribute("class","form-control");
        //file

        $form->addText('creatorFirstname', 'Křestní (první) jméno zadavatele')
            ->setDefaultValue($defaults['creatorFirstname'])
            ->setHtmlAttribute("class","form-control");
        $form->addText('creatorLastname', 'Příjmení (druhé jméno) zadavatele')
            ->setDefaultValue($defaults['creatorLastname'])
            ->setHtmlAttribute("class","form-control");
        $form->addEmail('creatorEmail', 'Kontaktní e-mail zadavatele')
            ->setDefaultValue($defaults['creatorEmail'])
            ->setHtmlAttribute("class", "form-control");
        $form->addText('creatorPhone', 'Kontaktní telefon zadavatele')
            ->setDefaultValue($defaults['creatorPhone'])
            ->setHtmlAttribute("class", "form-control");
        $form->addTextArea("creatorInfo", 'Další informace zadavatele')
            ->setDefaultValue($defaults['creatorInfo'])
            ->setHtmlAttribute("class", "form-control");


        $tags = $this->tagManager->getAllApplicableTags();

        if ($assignmentId != 0) {
            $defaults['profile'] = $this->assignmentManager->getAssignmentProfiles($row)->fetchPairs("id", "name");
        }

        $form->addMultiSelect('profile', 'Požadovaný profil týmu', $tags)
            ->setHtmlAttribute('class="selectpicker form-control" multiple data-live-search="true" title="Vyberte prosím předvolbu"')
            ->setHtmlAttribute("multiple=multiple")
            ->setHtmlAttribute("style= width:100%")->setDefaultValue(array_flip($defaults['profile']));

        $visibility = [
            Constants::$ASSIGNMENT_VISIBILITY_PUBLIC => 'Publikováno pro všechny',
            Constants::$ASSIGNMENT_VISIBILITY_MENTOR_ONLY => 'Viditelné pouze pro mentory',
            Constants::$ASSIGNMENT_VISIBILITY_HIDDEN => 'Skryté',
        ];

        $form->addSelect('visibility', 'Publikace zadání', $visibility)
            ->setDefaultValue($defaults['visibility'])
            ->setHtmlAttribute('class', 'form-control');

        $mentorsArray = array();

        $mentors = $this->database->table('USER')->where('role.name', RoleManager::$ROLE_MENTOR);

        $mentorsArray[0] = "Dosud nepřidělen";

        foreach ($mentors as $mentor) {
            $mentorsArray[$mentor->id] =Utils::getValidNameFromArgs($mentor->contact->firstname,$mentor->contact->lastname);
        }

        $form->addSelect('mentor', 'Přiřazený mentor', $mentorsArray)
            ->setDefaultValue($defaults['mentor'])
            ->setHtmlAttribute('class', 'form-control');

        $form->addHidden('userId', $userId);

        $periodsArray = array();
        $periods = $this->database->table('PERIOD');

        foreach ($periods as $period) {
            $periodsArray[$period->id] = $period->label;
        }


        $form->addSelect('period', 'Ročník aktuálního cyklu TSP', $periodsArray)
            ->setHtmlAttribute('class', 'form-control');

        $form->addUpload('pdfFile', 'PDF soubor s podrobným zadáním')
            ->addRule($form::MIME_TYPE, 'Požadovaný soubor musí být ve formátu PDF', ['application/x-pdf','application/pdf']);

        $form->addSubmit('send');

        $form->addHidden('assignmentId', $assignmentId);

        return $form;
    }

}